/*
	Copyright (C) 2020, 2021 Lewis ANESA.

	This file is part of ladspa-tool-kit

	ladspa-tool-kit is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	ladspa-tool-kit is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ladspa-tool-kit.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef __LTK_INSTANCE_H__
#define __LTK_INSTANCE_H__

#include <stdarg.h>
#include <stdint.h>

/* LTKctl_t corresponding macros */
#define LTKCTL_GET						0 << 0
#define LTKCTL_SET						1 << 0
#define LTKCTL_GET_ALL					~UINT64_C(1)
#define LTKCTL_ALLOC_ON_CLASS_CREATION	1 << 1
#define LTKCTL_SIZE_ON_CLASS_CREATION	1 << 2
#define LTKCTL_LOG_FD_ERROR				1 << 3
#define LTKCTL_LOG_FD_WARNING			1 << 4
#define LTKCTL_LOG_FD_INFO				1 << 5

/* LTKLog macros */
#define LTK_LOG_ERROR(format, ...)		(LTKLog(0, __FILE__, __LINE__, \
											__func__, format __VA_OPT__(,) \
											__VA_ARGS__))
#define LTK_LOG_WARNING(format, ...)	(LTKLog(1, __FILE__, __LINE__, \
											__func__, format __VA_OPT__(,) \
											__VA_ARGS__))
#define LTK_LOG_INFO(format, ...)		(LTKLog(2, __FILE__, __LINE__, \
											__func__, format __VA_OPT__(,) \
											__VA_ARGS__))

/* General class type definition */
typedef struct LTKctd_s LTKctd_t;
/* General method definition */
typedef void (*LTKfct_t)(void *, va_list);
/* General event handler including free */
typedef void (*LTKhdl_t)(void **, int);

typedef struct LTKctl_s {
	uint64_t _mask;
	uint64_t _def_buf_size;
	int _log_fd_error;
	int _log_fd_warning;
	int _log_fd_info;
} LTKctl_t;

LTKctd_t *LTKCtd(LTKctd_t *, LTKfct_t *, uint64_t, LTKhdl_t);
void *LTKAdd(LTKctd_t *);
int LTKClr(void *);
int LTKFre(void *);

int LTKRun(void *, uint64_t, ...);
int LTKFeh(LTKctd_t *, int, uint64_t, ...);

uint64_t *LTKNum(void *);
void *LTKOid(LTKctd_t *, uint64_t);

int LTKTrg(void *, int, uint32_t);
void *LTKWte(int *);

int LTKCtl(uint64_t, LTKctl_t *);
int LTKLog(unsigned int , char *, int, const char *, char *, ...);

#endif /* __LTK_INSTANCE_H__ */

